from PyObjCTools.TestSupport import TestCase, min_os_level
import objc
import GameController


class TestGCExtendedGamepad(TestCase):
    @min_os_level("10.9")
    def testClasses(self):
        self.assertIsInstance(GameController.GCExtendedGamepad, objc.objc_class)

    @min_os_level("10.9")
    def testMethods(self):
        self.assertResultIsBlock(
            GameController.GCExtendedGamepad.valueChangedHandler, b"v@@"
        )
        self.assertArgIsBlock(
            GameController.GCExtendedGamepad.setValueChangedHandler_, 0, b"v@@"
        )



from PyObjCTools.TestSupport import (
    TestCase,
    min_sdk_level,
)
import GameController


class TestGCRelativeInputHelper(GameController.NSObject):
    def deltaDidChangeHandler(self):
        return 1

    def setDeltaDidChangeHandler_(self, a):
        pass

    def delta(self):
        return 1

    def isAnalog(self):
        return 1

    def lastDeltaTimestamp(self):
        return 1

    def lastDeltaLatency(self):
        return 1


class TestGCRelativeInput(TestCase):
    @min_sdk_level("13.0")
    def test_protocols(self):
        self.assertProtocolExists("GCRelativeInput")

    def test_protocol_methods(self):
        self.assertResultIsBlock(
            TestGCRelativeInputHelper.deltaDidChangeHandler, b"v@@f"
        )
        self.assertArgIsBlock(
            TestGCRelativeInputHelper.setDeltaDidChangeHandler_, 0, b"v@@f"
        )

        self.assertResultHasType(TestGCRelativeInputHelper.delta, b"f")
        self.assertResultIsBOOL(TestGCRelativeInputHelper.isAnalog)
        self.assertResultHasType(TestGCRelativeInputHelper.lastDeltaTimestamp, b"d")
        self.assertResultHasType(TestGCRelativeInputHelper.lastDeltaLatency, b"d")



from PyObjCTools.TestSupport import (
    TestCase,
    min_sdk_level,
)
import GameController


class TestGCDevicePhysicalInputStateDiff(TestCase):
    def test_constants(self):
        self.assertIsEnumType(GameController.GCDevicePhysicalInputElementChange)
        self.assertEqual(GameController.GCDevicePhysicalInputElementUnknownChange, -1)
        self.assertEqual(GameController.GCDevicePhysicalInputElementNoChange, 0)
        self.assertEqual(GameController.GCDevicePhysicalInputElementChanged, 1)

    @min_sdk_level("13.0")
    def test_protocols(self):
        self.assertProtocolExists("GCDevicePhysicalInputStateDiff")



from PyObjCTools.TestSupport import (
    TestCase,
    min_sdk_level,
)
import objc
import GameController


class TestGCSwitchPositionInputHelper(GameController.NSObject):
    def positionDidChangeHandler(self):
        return 1

    def setPositionDidChangeHandler_(self, a):
        pass

    def position(self):
        return 1

    def positionRange(self):
        return 1

    def isSequential(self):
        return 1

    def canWrap(self):
        return 1

    def lastPositionTimestamp(self):
        return 1

    def lastPositionLatency(self):
        return 1


class TestGCSwitchPositionInput(TestCase):
    @min_sdk_level("13.0")
    def test_protocols(self):
        self.assertProtocolExists("GCSwitchPositionInput")

    def test_protocol_methhods(self):
        self.assertResultIsBlock(
            GameController.TestGCSwitchPositionInputHelper.positionDidChangeHandler,
            b"v@@" + objc._C_NSInteger,
        )
        self.assertArgIsBlock(
            GameController.TestGCSwitchPositionInputHelper.setPositionDidChangeHandler_,
            0,
            b"v@@" + objc._C_NSInteger,
        )

        self.assertResultHasType(
            GameController.TestGCSwitchPositionInputHelper.position, objc._C_NSInteger
        )
        self.assertResultHasType(
            GameController.TestGCSwitchPositionInputHelper.positionRange,
            GameController.NSRange.__typestr__,
        )
        self.assertResultIsBOOL(
            GameController.TestGCSwitchPositionInputHelper.isSequential
        )
        self.assertResultIsBOOL(GameController.TestGCSwitchPositionInputHelper.canWrap)
        self.assertResultHasType(
            GameController.TestGCSwitchPositionInputHelper.lastPositionTimestamp, b"d"
        )
        self.assertResultHasType(
            GameController.TestGCSwitchPositionInputHelper.lastPositionLatency, b"d"
        )



from PyObjCTools.TestSupport import (
    TestCase,
    min_sdk_level,
)
import GameController


class TestGCPressedStateInputHelper(GameController.NSObject):
    def pressedDidChangeHandler(self):
        return 1

    def setPressedDidChangeHandler_(self, a):
        pass

    def isPressed(self):
        return 1

    def lastPressedStateTimestamp(self):
        return 1

    def lastPressedStateLatency(self):
        return 1


class TestGCPressedStateInput(TestCase):
    @min_sdk_level("13.0")
    def test_protocols(self):
        self.assertProtocolExists("GCPressedStateInput")

    def test_protocol_methods(self):
        self.assertResultIsBlock(
            TestGCPressedStateInputHelper.pressedDidChangeHandler, b"v@@Z"
        )
        self.assertArgIsBlock(
            TestGCPressedStateInputHelper.setPressedDidChangeHandler_, 0, b"v@@Z"
        )

        self.assertResultIsBOOL(TestGCPressedStateInputHelper.isPressed)
        self.assertResultHasType(
            TestGCPressedStateInputHelper.lastPressedStateTimestamp, b"d"
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import objc
import GameController


class TestGCControllerTouchpad(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(GameController.GCTouchState)

    def test_constants(self):
        self.assertEqual(GameController.GCTouchStateUp, 0)
        self.assertEqual(GameController.GCTouchStateDown, 1)
        self.assertEqual(GameController.GCTouchStateMoving, 2)

    @min_os_level("11.0")
    def test_methods11_0(self):
        self.assertResultIsBlock(
            GameController.GCControllerTouchpad.touchDown, b"vfff" + objc._C_NSBOOL
        )
        self.assertArgIsBlock(
            GameController.GCControllerTouchpad.setTouchDown_,
            0,
            b"vfff" + objc._C_NSBOOL,
        )

        self.assertResultIsBlock(
            GameController.GCControllerTouchpad.touchMoved, b"vfff" + objc._C_NSBOOL
        )
        self.assertArgIsBlock(
            GameController.GCControllerTouchpad.setTouchMoved_,
            0,
            b"vfff" + objc._C_NSBOOL,
        )

        self.assertResultIsBlock(
            GameController.GCControllerTouchpad.touchUp, b"vfff" + objc._C_NSBOOL
        )
        self.assertArgIsBlock(
            GameController.GCControllerTouchpad.setTouchUp_, 0, b"vfff" + objc._C_NSBOOL
        )

        self.assertResultIsBOOL(
            GameController.GCControllerTouchpad.reportsAbsoluteTouchSurfaceValues
        )
        self.assertArgIsBOOL(
            GameController.GCControllerTouchpad.setReportsAbsoluteTouchSurfaceValues_, 0
        )

        self.assertArgIsBOOL(
            GameController.GCControllerTouchpad.setValueForXAxis_yAxis_touchDown_buttonValue_,
            2,
        )



from PyObjCTools.TestSupport import (
    TestCase,
    min_os_level,
)
import GameController


class TestGCDirectionalGamepad(TestCase):
    @min_os_level("11.3")
    def testConstants11_3(self):
        self.assertIsInstance(GameController.GCInputDirectionalDpad, str)
        self.assertIsInstance(GameController.GCInputDirectionalCardinalDpad, str)



from PyObjCTools.TestSupport import (
    TestCase,
    min_os_level,
)
import GameController


class TestGCRacingWheel(TestCase):
    @min_os_level("13.0")
    def test_constants13_0(self):
        self.assertIsInstance(GameController.GCRacingWheelDidConnectNotification, str)
        self.assertIsInstance(
            GameController.GCRacingWheelDidDisconnectNotification, str
        )

    @min_os_level("13.0")
    def test_methods13_0(self):
        self.assertResultIsBOOL(GameController.GCRacingWheel.acquireDeviceWithError_)
        self.assertArgIsOut(GameController.GCRacingWheel.acquireDeviceWithError_, 0)

        self.assertResultIsBOOL(GameController.GCRacingWheel.isAcquired)
        self.assertResultIsBOOL(GameController.GCRacingWheel.isSnapshot)



from PyObjCTools.TestSupport import (
    TestCase,
    min_sdk_level,
)
import GameController


class TestGCDevicePhysicalInputHelper(GameController.NSObject):
    def elementValueDidChangeHandler(self):
        return 1

    def setElementValueDidChangeHandler_(self, a):
        pass

    def inputStateAvailableHandler(self):
        return 1

    def setInputStateAvailableHandler_(self, a):
        pass

    def inputStateQueueDepth(self):
        return 1

    def setInputStateQueueDepth_(self, a):
        pass


class TestGCDevicePhysicalInput(TestCase):
    @min_sdk_level("13.0")
    def test_protocols(self):
        self.assertProtocolExists("GCDevicePhysicalInput")

    def test_protocol_methods(self):
        self.assertResultIsBlock(
            TestGCDevicePhysicalInputHelper.elementValueDidChangeHandler, b"v@@"
        )
        self.assertArgIsBlock(
            TestGCDevicePhysicalInputHelper.setElementValueDidChangeHandler_, 0, b"v@@"
        )

        self.assertResultIsBlock(
            TestGCDevicePhysicalInputHelper.inputStateAvailableHandler, b"v@"
        )
        self.assertArgIsBlock(
            TestGCDevicePhysicalInputHelper.setInputStateAvailableHandler_, 0, b"v@"
        )



from PyObjCTools.TestSupport import (
    TestCase,
)
import GameController


class GCSyntheticDeviceKeys(TestCase):
    def test_constants(self):
        self.assertEqual(
            GameController.kIOHIDGCSyntheticDeviceKey, b"GCSyntheticDevice"
        )



from PyObjCTools.TestSupport import (
    TestCase,
    min_sdk_level,
)
import GameController


class TestGCAxisInputHelper(GameController.NSObject):
    def valueDidChangeHandler(self):
        return 1

    def setValueDidChangeHandler_(self, a):
        pass

    def value(self):
        return 1

    def isAnalog(self):
        return 1

    def canWrap(self):
        return 1

    def lastValueTimestamp(self):
        return 1

    def lastValueLatency(self):
        return 1


class TestGCAxisInput(TestCase):
    @min_sdk_level("13.0")
    def test_protocols(self):
        self.assertProtocolExists("GCAxisInput")

    def test_protocol_methods(self):
        self.assertResultIsBlock(TestGCAxisInputHelper.valueDidChangeHandler, b"v@@f")

        self.assertArgIsBlock(
            TestGCAxisInputHelper.setValueDidChangeHandler_, 0, b"v@@f"
        )

        self.assertResultHasType(TestGCAxisInputHelper.value, b"f")

        self.assertResultIsBOOL(TestGCAxisInputHelper.isAnalog)
        self.assertResultIsBOOL(TestGCAxisInputHelper.canWrap)
        self.assertResultHasType(TestGCAxisInputHelper.lastValueTimestamp, b"d")
        self.assertResultHasType(TestGCAxisInputHelper.lastValueLatency, b"d")



from PyObjCTools.TestSupport import TestCase, min_os_level
import objc
import GameController


class TestGCGamepadSnapshot(TestCase):
    @min_os_level("10.9")
    def testClasses(self):
        self.assertIsInstance(GameController.GCGamepadSnapshot, objc.objc_class)

    @min_os_level("10.9")
    def testStructs(self):
        self.assertEqual(GameController.GCGamepadSnapShotDataV100.__struct_pack__, 1)

        v = GameController.GCGamepadSnapShotDataV100()
        self.assertIsInstance(v.version, int)
        self.assertIsInstance(v.size, int)
        self.assertIsInstance(v.dpadX, float)
        self.assertIsInstance(v.dpadY, float)
        self.assertIsInstance(v.buttonA, float)
        self.assertIsInstance(v.buttonB, float)
        self.assertIsInstance(v.buttonX, float)
        self.assertIsInstance(v.buttonY, float)
        self.assertIsInstance(v.leftShoulder, float)
        self.assertIsInstance(v.rightShoulder, float)
        self.assertPickleRoundTrips(v)

    @min_os_level("10.9")
    def testFunctions(self):
        self.assertResultIsBOOL(GameController.GCGamepadSnapShotDataV100FromNSData)
        self.assertArgIsOut(GameController.GCGamepadSnapShotDataV100FromNSData, 0)
        self.assertArgIsIn(GameController.NSDataFromGCGamepadSnapShotDataV100, 0)



from PyObjCTools.TestSupport import TestCase, min_os_level
import GameController


class TestGCDeviceHaptics(TestCase):
    def test_typed_enum(self):
        self.assertIsTypedEnum(GameController.GCHapticsLocality, str)

    @min_os_level("11.0")
    def test_constants11_0(self):
        self.assertIsInstance(GameController.GCHapticsLocalityDefault, str)
        self.assertIsInstance(GameController.GCHapticsLocalityAll, str)
        self.assertIsInstance(GameController.GCHapticsLocalityHandles, str)
        self.assertIsInstance(GameController.GCHapticsLocalityLeftHandle, str)
        self.assertIsInstance(GameController.GCHapticsLocalityRightHandle, str)
        self.assertIsInstance(GameController.GCHapticsLocalityTriggers, str)
        self.assertIsInstance(GameController.GCHapticsLocalityLeftTrigger, str)
        self.assertIsInstance(GameController.GCHapticsLocalityRightTrigger, str)

        self.assertIsInstance(GameController.GCHapticDurationInfinite, float)



from PyObjCTools.TestSupport import TestCase, min_os_level
import GameController


class TestGCPhysicalInputProfile(TestCase):
    @min_os_level("12.0")
    def testMethods(self):
        self.assertResultIsBOOL(
            GameController.GCPhysicalInputProfile.hasRemappedElements
        )

    @min_os_level("13.0")
    def testMethods13_0(self):
        self.assertResultIsBlock(
            GameController.GCPhysicalInputProfile.valueDidChangeHandler, b"v@@"
        )
        self.assertArgIsBlock(
            GameController.GCPhysicalInputProfile.setValueDidChangeHandler_, 0, b"v@@"
        )



from PyObjCTools.TestSupport import TestCase
import GameController


class TestGCDualSenseAdaptiveTrigger(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(GameController.GCDualSenseAdaptiveTriggerMode)
        self.assertIsEnumType(GameController.GCDualSenseAdaptiveTriggerStatus)

    def test_constants(self):
        self.assertEqual(
            GameController.GCDualSenseAdaptiveTriggerDiscretePositionCount, 10
        )

        self.assertEqual(GameController.GCDualSenseAdaptiveTriggerModeOff, 0)
        self.assertEqual(GameController.GCDualSenseAdaptiveTriggerModeFeedback, 1)
        self.assertEqual(GameController.GCDualSenseAdaptiveTriggerModeWeapon, 2)
        self.assertEqual(GameController.GCDualSenseAdaptiveTriggerModeVibration, 3)
        self.assertEqual(GameController.GCDualSenseAdaptiveTriggerModeSlopeFeedback, 4)

        self.assertEqual(GameController.GCDualSenseAdaptiveTriggerStatusUnknown, -1)
        self.assertEqual(
            GameController.GCDualSenseAdaptiveTriggerStatusFeedbackNoLoad, 0
        )
        self.assertEqual(
            GameController.GCDualSenseAdaptiveTriggerStatusFeedbackLoadApplied, 1
        )
        self.assertEqual(GameController.GCDualSenseAdaptiveTriggerStatusWeaponReady, 2)
        self.assertEqual(GameController.GCDualSenseAdaptiveTriggerStatusWeaponFiring, 3)
        self.assertEqual(GameController.GCDualSenseAdaptiveTriggerStatusWeaponFired, 4)
        self.assertEqual(
            GameController.GCDualSenseAdaptiveTriggerStatusVibrationNotVibrating, 5
        )
        self.assertEqual(
            GameController.GCDualSenseAdaptiveTriggerStatusVibrationIsVibrating, 6
        )
        self.assertEqual(
            GameController.GCDualSenseAdaptiveTriggerStatusSlopeFeedbackReady, 7
        )
        self.assertEqual(
            GameController.GCDualSenseAdaptiveTriggerStatusSlopeFeedbackApplyingLoad, 8
        )
        self.assertEqual(
            GameController.GCDualSenseAdaptiveTriggerStatusSlopeFeedbackFinished, 9
        )

    def test_structs(self):
        v = GameController.GCDualSenseAdaptiveTriggerPositionalAmplitudes()
        self.assertIs(v.values, None)

        v = GameController.GCDualSenseAdaptiveTriggerPositionalResistiveStrengths()
        self.assertIs(v.values, None)



from PyObjCTools.TestSupport import TestCase, min_os_level
import objc
import GameController


class TestGCControllerButtonInput(TestCase):
    @min_os_level("10.9")
    def testClasses(self):
        self.assertIsInstance(GameController.GCControllerButtonInput, objc.objc_class)

    @min_os_level("10.9")
    def testMethods(self):
        self.assertResultIsBlock(
            GameController.GCControllerButtonInput.valueChangedHandler,
            b"v@f" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            GameController.GCControllerButtonInput.setValueChangedHandler_,
            0,
            b"v@f" + objc._C_NSBOOL,
        )

        self.assertResultIsBOOL(GameController.GCControllerButtonInput.isPressed)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBlock(
            GameController.GCControllerButtonInput.pressedChangedHandler,
            b"v@f" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            GameController.GCControllerButtonInput.setPressedChangedHandler_,
            0,
            b"v@f" + objc._C_NSBOOL,
        )

    @min_os_level("11.0")
    def testMethods11_0(self):
        self.assertResultIsBlock(
            GameController.GCControllerButtonInput.touchedChangedHandler,
            b"v@f" + objc._C_NSBOOL + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            GameController.GCControllerButtonInput.setTouchedChangedHandler_,
            0,
            b"v@f" + objc._C_NSBOOL + objc._C_NSBOOL,
        )

        self.assertResultIsBOOL(GameController.GCControllerButtonInput.isTouched)



from PyObjCTools.TestSupport import (
    TestCase,
    min_sdk_level,
)
import objc
import GameController


class TestGCDevicePhysicalInputStateHelper(GameController.NSObject):
    def lastEventTimestamp(self):
        return 1

    def lastEventLatency(self):
        return 1


class TestGCDevicePhysicalInputState(TestCase):
    @min_sdk_level("13.0")
    def test_protocols(self):
        self.assertProtocolExists("GCDevicePhysicalInputState")

    def test_protocol_methods(self):
        self.assertResultHasType(
            TestGCDevicePhysicalInputStateHelper.lastEventTimestamp, objc._C_DBL
        )
        self.assertResultHasType(
            TestGCDevicePhysicalInputStateHelper.lastEventLatency, objc._C_DBL
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import objc

import GameController


class TestGCGamePad(TestCase):
    @min_os_level("10.9")
    def testClasses(self):
        self.assertIsInstance(GameController.GCGamepad, objc.objc_class)

    @min_os_level("10.9")
    def testMethods(self):
        self.assertResultIsBlock(GameController.GCGamepad.valueChangedHandler, b"v@@")
        self.assertArgIsBlock(
            GameController.GCGamepad.setValueChangedHandler_, 0, b"v@@"
        )



from PyObjCTools.TestSupport import (
    TestCase,
    min_os_level,
    os_release,
    expectedFailureIf,
)
import objc
import GameController


class TestGCMicroGamepadSnapshot(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(GameController.GCMicroGamepadSnapshotDataVersion)

    @expectedFailureIf(os_release() == "10.11")
    @min_os_level("10.11")
    def testClasses(self):
        self.assertIsInstance(GameController.GCMicroGamepadSnapshot, objc.objc_class)

    @min_os_level("10.11")
    def testStructs(self):
        self.assertEqual(
            GameController.GCMicroGamepadSnapShotDataV100.__struct_pack__, 1
        )

        v = GameController.GCMicroGamepadSnapShotDataV100()
        self.assertIsInstance(v.version, int)
        self.assertIsInstance(v.size, int)
        self.assertIsInstance(v.dpadX, float)
        self.assertIsInstance(v.dpadY, float)
        self.assertIsInstance(v.buttonA, float)
        self.assertIsInstance(v.buttonX, float)
        self.assertPickleRoundTrips(v)

    @min_os_level("10.14")
    def testStructs10_14_4(self):
        self.assertEqual(GameController.GCMicroGamepadSnapshotData.__struct_pack__, 1)

        v = GameController.GCMicroGamepadSnapshotData()
        self.assertIsInstance(v.version, int)
        self.assertIsInstance(v.size, int)
        self.assertIsInstance(v.dpadX, float)
        self.assertIsInstance(v.dpadY, float)
        self.assertIsInstance(v.buttonA, float)
        self.assertIsInstance(v.buttonX, float)

    @expectedFailureIf(os_release().rsplit(".", 1)[0] in ("10.9", "10.10", "10.11"))
    @min_os_level("10.9")
    def testFunctions(self):
        self.assertResultIsBOOL(GameController.GCMicroGamepadSnapShotDataV100FromNSData)
        self.assertArgIsOut(GameController.GCMicroGamepadSnapShotDataV100FromNSData, 0)
        self.assertArgIsIn(GameController.NSDataFromGCMicroGamepadSnapShotDataV100, 0)

    @min_os_level("10.14.4")
    def testFunctions10_14_4(self):
        self.assertResultIsBOOL(GameController.GCMicroGamepadSnapshotDataFromNSData)
        self.assertArgIsOut(GameController.GCMicroGamepadSnapshotDataFromNSData, 0)
        self.assertArgIsIn(GameController.NSDataFromGCMicroGamepadSnapshotData, 0)

    def test_constants(self):
        self.assertEqual(GameController.GCMicroGamepadSnapshotDataVersion1, 0x0100)

    @min_os_level("10.14.4")
    def test_constants10_14_4(self):
        self.assertIsInstance(
            GameController.GCCurrentMicroGamepadSnapshotDataVersion, int
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import GameController


class TestGCMouse(TestCase):
    @min_os_level("11.0")
    def test_constants11_0(self):
        self.assertIsInstance(GameController.GCMouseDidConnectNotification, str)
        self.assertIsInstance(GameController.GCMouseDidDisconnectNotification, str)

        self.assertIsInstance(GameController.GCMouseDidBecomeCurrentNotification, str)
        self.assertIsInstance(
            GameController.GCMouseDidStopBeingCurrentNotification, str
        )



from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import GameController


class TestGCInputNames(TestCase):
    @min_os_level("11.0")
    def test_constants11_0(self):
        self.assertIsInstance(GameController.GCInputButtonA, str)
        self.assertIsInstance(GameController.GCInputButtonB, str)
        self.assertIsInstance(GameController.GCInputButtonX, str)
        self.assertIsInstance(GameController.GCInputButtonY, str)

        self.assertIsInstance(GameController.GCInputDirectionPad, str)
        self.assertIsInstance(GameController.GCInputLeftThumbstick, str)
        self.assertIsInstance(GameController.GCInputRightThumbstick, str)

        self.assertIsInstance(GameController.GCInputLeftShoulder, str)
        self.assertIsInstance(GameController.GCInputRightShoulder, str)
        self.assertIsInstance(GameController.GCInputLeftTrigger, str)
        self.assertIsInstance(GameController.GCInputRightTrigger, str)
        self.assertIsInstance(GameController.GCInputLeftThumbstickButton, str)
        self.assertIsInstance(GameController.GCInputRightThumbstickButton, str)

        self.assertIsInstance(GameController.GCInputButtonHome, str)
        self.assertIsInstance(GameController.GCInputButtonMenu, str)
        self.assertIsInstance(GameController.GCInputButtonOptions, str)

        self.assertIsInstance(GameController.GCInputXboxPaddleOne, str)
        self.assertIsInstance(GameController.GCInputXboxPaddleTwo, str)
        self.assertIsInstance(GameController.GCInputXboxPaddleThree, str)
        self.assertIsInstance(GameController.GCInputXboxPaddleFour, str)

        self.assertIsInstance(GameController.GCInputDualShockTouchpadOne, str)
        self.assertIsInstance(GameController.GCInputDualShockTouchpadTwo, str)
        self.assertIsInstance(GameController.GCInputDualShockTouchpadButton, str)

    @min_os_level("12.0")
    def test_constants12_0(self):
        self.assertIsInstance(GameController.GCInputButtonShare, str)

    @min_os_level("13.0")
    def test_constants13_0(self):
        self.assertIsInstance(GameController.GCInputSteeringWheel, str)
        self.assertIsInstance(GameController.GCInputShifter, str)
        self.assertIsInstance(GameController.GCInputPedalAccelerator, str)
        self.assertIsInstance(GameController.GCInputPedalBrake, str)
        self.assertIsInstance(GameController.GCInputPedalClutch, str)
        self.assertIsInstance(GameController.GCInputLeftPaddle, str)
        self.assertIsInstance(GameController.GCInputRightPaddle, str)

    def test_enum_types(self):
        self.assertIsTypedEnum(GameController.GCInputElementName, str)
        self.assertIsTypedEnum(GameController.GCInputButtonName, str)
        self.assertIsTypedEnum(GameController.GCInputAxisName, str)
        self.assertIsTypedEnum(GameController.GCInputDirectionPadName, str)

    @min_sdk_level("13.0")
    def test_protocols(self):
        pass

        # These are compile time only protocols:
        # self.assertProtocolExists("GCPhysicalInputElementName")
        # self.assertProtocolExists("GCButtonElementName")
        # self.assertProtocolExists("GCAxisElementName")
        # self.assertProtocolExists("GCSwitchElementName")
        # self.assertProtocolExists("GCDirectionPadElementName")



# This file is generated by objective.metadata
#
# Last update: Fri Feb 17 21:48:40 2023
#
# flake8: noqa

import objc, sys
from typing import NewType

if sys.maxsize > 2**32:

    def sel32or64(a, b):
        return b

else:

    def sel32or64(a, b):
        return a


if objc.arch == "arm64":

    def selAorI(a, b):
        return a

else:

    def selAorI(a, b):
        return b


misc = {}
misc.update(
    {
        "GCMicroGamepadSnapshotData": objc.createStructType(
            "GameController.GCMicroGamepadSnapshotData",
            b"{_GCMicroGamepadSnapshotData=SSffff}",
            ["version", "size", "dpadX", "dpadY", "buttonA", "buttonX"],
            None,
            1,
        ),
        "GCQuaternion": objc.createStructType(
            "GameController.GCQuaternion", b"{GCQuaternion=dddd}", ["x", "y", "z", "w"]
        ),
        "GCExtendedGamepadValueChangedHandler": objc.createStructType(
            "GameController.GCExtendedGamepadValueChangedHandler",
            b"{_GCGamepadSnapShotDataV100=SSffffffff}",
            [
                "version",
                "size",
                "dpadX",
                "dpadY",
                "buttonA",
                "buttonB",
                "buttonX",
                "buttonY",
                "leftShoulder",
                "rightShoulder",
            ],
        ),
        "GCAcceleration": objc.createStructType(
            "GameController.GCAcceleration", b"{_GCAcceleration=ddd}", ["x", "y", "z"]
        ),
        "GCGamepadSnapShotDataV100": objc.createStructType(
            "GameController.GCGamepadSnapShotDataV100",
            b"{_GCGamepadSnapShotDataV100=SSffffffff}",
            [
                "version",
                "size",
                "dpadX",
                "dpadY",
                "buttonA",
                "buttonB",
                "buttonX",
                "buttonY",
                "leftShoulder",
                "rightShoulder",
            ],
            None,
            1,
        ),
        "GCEulerAngles": objc.createStructType(
            "GameController.GCEulerAngles",
            b"{_GCEulerAngles=ddd}",
            ["pitch", "yaw", "roll"],
        ),
        "GCDualSenseAdaptiveTriggerPositionalResistiveStrengths": objc.createStructType(
            "GameController.GCDualSenseAdaptiveTriggerPositionalResistiveStrengths",
            b"{_GCDualSenseAdaptiveTriggerPositionalResistiveStrengths=[10f]}",
            ["values"],
        ),
        "GCExtendedGamepadSnapShotDataV100": objc.createStructType(
            "GameController.GCExtendedGamepadSnapShotDataV100",
            b"{_GCExtendedGamepadSnapShotDataV100=SSffffffffffffff}",
            [
                "version",
                "size",
                "dpadX",
                "dpadY",
                "buttonA",
                "buttonB",
                "buttonX",
                "buttonY",
                "leftShoulder",
                "rightShoulder",
                "leftThumbstickX",
                "leftThumbstickY",
                "rightThumbstickX",
                "rightThumbstickY",
                "leftTrigger",
                "rightTrigger",
            ],
            None,
            1,
        ),
        "GCDualSenseAdaptiveTriggerPositionalAmplitudes": objc.createStructType(
            "GameController.GCDualSenseAdaptiveTriggerPositionalAmplitudes",
            b"{_GCDualSenseAdaptiveTriggerPositionalAmplitudes=[10f]}",
            ["values"],
        ),
        "GCRotationRate": objc.createStructType(
            "GameController.GCRotationRate", b"{_GCRotationRate=ddd}", ["x", "y", "z"]
        ),
        "GCMicroGamepadSnapShotDataV100": objc.createStructType(
            "GameController.GCMicroGamepadSnapShotDataV100",
            b"{_GCMicroGamepadSnapShotDataV100=SSffff}",
            ["version", "size", "dpadX", "dpadY", "buttonA", "buttonX"],
            None,
            1,
        ),
        "GCExtendedGamepadSnapshotData": objc.createStructType(
            "GameController.GCExtendedGamepadSnapshotData",
            b"{_GCExtendedGamepadSnapshotData=SSffffffffffffffZZZ}",
            [
                "version",
                "size",
                "dpadX",
                "dpadY",
                "buttonA",
                "buttonB",
                "buttonX",
                "buttonY",
                "leftShoulder",
                "rightShoulder",
                "leftThumbstickX",
                "leftThumbstickY",
                "rightThumbstickX",
                "rightThumbstickY",
                "leftTrigger",
                "rightTrigger",
                "supportsClickableThumbsticks",
                "leftThumbstickButton",
                "rightThumbstickButton",
            ],
            None,
            1,
        ),
    }
)
constants = """$GCControllerDidBecomeCurrentNotification$GCControllerDidConnectNotification$GCControllerDidDisconnectNotification$GCControllerDidStopBeingCurrentNotification$GCControllerUserCustomizationsDidChangeNotification$GCCurrentExtendedGamepadSnapshotDataVersion@q$GCCurrentMicroGamepadSnapshotDataVersion@q$GCHapticDurationInfinite@f$GCHapticsLocalityAll$GCHapticsLocalityDefault$GCHapticsLocalityHandles$GCHapticsLocalityLeftHandle$GCHapticsLocalityLeftTrigger$GCHapticsLocalityRightHandle$GCHapticsLocalityRightTrigger$GCHapticsLocalityTriggers$GCInputButtonA$GCInputButtonB$GCInputButtonHome$GCInputButtonMenu$GCInputButtonOptions$GCInputButtonShare$GCInputButtonX$GCInputButtonY$GCInputDirectionPad$GCInputDirectionalCardinalDpad$GCInputDirectionalCenterButton$GCInputDirectionalDpad$GCInputDirectionalTouchSurfaceButton$GCInputDualShockTouchpadButton$GCInputDualShockTouchpadOne$GCInputDualShockTouchpadTwo$GCInputLeftPaddle$GCInputLeftShoulder$GCInputLeftThumbstick$GCInputLeftThumbstickButton$GCInputLeftTrigger$GCInputMicroGamepadButtonA$GCInputMicroGamepadButtonMenu$GCInputMicroGamepadButtonX$GCInputMicroGamepadDpad$GCInputPedalAccelerator$GCInputPedalBrake$GCInputPedalClutch$GCInputRightPaddle$GCInputRightShoulder$GCInputRightThumbstick$GCInputRightThumbstickButton$GCInputRightTrigger$GCInputShifter$GCInputSteeringWheel$GCInputXboxPaddleFour$GCInputXboxPaddleOne$GCInputXboxPaddleThree$GCInputXboxPaddleTwo$GCKeyA$GCKeyApplication$GCKeyB$GCKeyBackslash$GCKeyC$GCKeyCapsLock$GCKeyCloseBracket$GCKeyCodeApplication@q$GCKeyCodeBackslash@q$GCKeyCodeCapsLock@q$GCKeyCodeCloseBracket@q$GCKeyCodeComma@q$GCKeyCodeDeleteForward@q$GCKeyCodeDeleteOrBackspace@q$GCKeyCodeDownArrow@q$GCKeyCodeEight@q$GCKeyCodeEnd@q$GCKeyCodeEqualSign@q$GCKeyCodeEscape@q$GCKeyCodeF1@q$GCKeyCodeF10@q$GCKeyCodeF11@q$GCKeyCodeF12@q$GCKeyCodeF13@q$GCKeyCodeF14@q$GCKeyCodeF15@q$GCKeyCodeF16@q$GCKeyCodeF17@q$GCKeyCodeF18@q$GCKeyCodeF19@q$GCKeyCodeF2@q$GCKeyCodeF20@q$GCKeyCodeF3@q$GCKeyCodeF4@q$GCKeyCodeF5@q$GCKeyCodeF6@q$GCKeyCodeF7@q$GCKeyCodeF8@q$GCKeyCodeF9@q$GCKeyCodeFive@q$GCKeyCodeFour@q$GCKeyCodeGraveAccentAndTilde@q$GCKeyCodeHome@q$GCKeyCodeHyphen@q$GCKeyCodeInsert@q$GCKeyCodeInternational1@q$GCKeyCodeInternational2@q$GCKeyCodeInternational3@q$GCKeyCodeInternational4@q$GCKeyCodeInternational5@q$GCKeyCodeInternational6@q$GCKeyCodeInternational7@q$GCKeyCodeInternational8@q$GCKeyCodeInternational9@q$GCKeyCodeKeyA@q$GCKeyCodeKeyB@q$GCKeyCodeKeyC@q$GCKeyCodeKeyD@q$GCKeyCodeKeyE@q$GCKeyCodeKeyF@q$GCKeyCodeKeyG@q$GCKeyCodeKeyH@q$GCKeyCodeKeyI@q$GCKeyCodeKeyJ@q$GCKeyCodeKeyK@q$GCKeyCodeKeyL@q$GCKeyCodeKeyM@q$GCKeyCodeKeyN@q$GCKeyCodeKeyO@q$GCKeyCodeKeyP@q$GCKeyCodeKeyQ@q$GCKeyCodeKeyR@q$GCKeyCodeKeyS@q$GCKeyCodeKeyT@q$GCKeyCodeKeyU@q$GCKeyCodeKeyV@q$GCKeyCodeKeyW@q$GCKeyCodeKeyX@q$GCKeyCodeKeyY@q$GCKeyCodeKeyZ@q$GCKeyCodeKeypad0@q$GCKeyCodeKeypad1@q$GCKeyCodeKeypad2@q$GCKeyCodeKeypad3@q$GCKeyCodeKeypad4@q$GCKeyCodeKeypad5@q$GCKeyCodeKeypad6@q$GCKeyCodeKeypad7@q$GCKeyCodeKeypad8@q$GCKeyCodeKeypad9@q$GCKeyCodeKeypadAsterisk@q$GCKeyCodeKeypadEnter@q$GCKeyCodeKeypadEqualSign@q$GCKeyCodeKeypadHyphen@q$GCKeyCodeKeypadNumLock@q$GCKeyCodeKeypadPeriod@q$GCKeyCodeKeypadPlus@q$GCKeyCodeKeypadSlash@q$GCKeyCodeLANG1@q$GCKeyCodeLANG2@q$GCKeyCodeLANG3@q$GCKeyCodeLANG4@q$GCKeyCodeLANG5@q$GCKeyCodeLANG6@q$GCKeyCodeLANG7@q$GCKeyCodeLANG8@q$GCKeyCodeLANG9@q$GCKeyCodeLeftAlt@q$GCKeyCodeLeftArrow@q$GCKeyCodeLeftControl@q$GCKeyCodeLeftGUI@q$GCKeyCodeLeftShift@q$GCKeyCodeNine@q$GCKeyCodeNonUSBackslash@q$GCKeyCodeNonUSPound@q$GCKeyCodeOne@q$GCKeyCodeOpenBracket@q$GCKeyCodePageDown@q$GCKeyCodePageUp@q$GCKeyCodePause@q$GCKeyCodePeriod@q$GCKeyCodePower@q$GCKeyCodePrintScreen@q$GCKeyCodeQuote@q$GCKeyCodeReturnOrEnter@q$GCKeyCodeRightAlt@q$GCKeyCodeRightArrow@q$GCKeyCodeRightControl@q$GCKeyCodeRightGUI@q$GCKeyCodeRightShift@q$GCKeyCodeScrollLock@q$GCKeyCodeSemicolon@q$GCKeyCodeSeven@q$GCKeyCodeSix@q$GCKeyCodeSlash@q$GCKeyCodeSpacebar@q$GCKeyCodeTab@q$GCKeyCodeThree@q$GCKeyCodeTwo@q$GCKeyCodeUpArrow@q$GCKeyCodeZero@q$GCKeyComma$GCKeyD$GCKeyDeleteForward$GCKeyDeleteOrBackspace$GCKeyDownArrow$GCKeyE$GCKeyEight$GCKeyEnd$GCKeyEqualSign$GCKeyEscape$GCKeyF$GCKeyF1$GCKeyF10$GCKeyF11$GCKeyF12$GCKeyF13$GCKeyF14$GCKeyF15$GCKeyF16$GCKeyF17$GCKeyF18$GCKeyF19$GCKeyF2$GCKeyF20$GCKeyF3$GCKeyF4$GCKeyF5$GCKeyF6$GCKeyF7$GCKeyF8$GCKeyF9$GCKeyFive$GCKeyFour$GCKeyG$GCKeyGraveAccentAndTilde$GCKeyH$GCKeyHome$GCKeyHyphen$GCKeyI$GCKeyInsert$GCKeyInternational1$GCKeyInternational2$GCKeyInternational3$GCKeyInternational4$GCKeyInternational5$GCKeyInternational6$GCKeyInternational7$GCKeyInternational8$GCKeyInternational9$GCKeyJ$GCKeyK$GCKeyKeypad0$GCKeyKeypad1$GCKeyKeypad2$GCKeyKeypad3$GCKeyKeypad4$GCKeyKeypad5$GCKeyKeypad6$GCKeyKeypad7$GCKeyKeypad8$GCKeyKeypad9$GCKeyKeypadAsterisk$GCKeyKeypadEnter$GCKeyKeypadEqualSign$GCKeyKeypadHyphen$GCKeyKeypadNumLock$GCKeyKeypadPeriod$GCKeyKeypadPlus$GCKeyKeypadSlash$GCKeyL$GCKeyLANG1$GCKeyLANG2$GCKeyLANG3$GCKeyLANG4$GCKeyLANG5$GCKeyLANG6$GCKeyLANG7$GCKeyLANG8$GCKeyLANG9$GCKeyLeftAlt$GCKeyLeftArrow$GCKeyLeftControl$GCKeyLeftGUI$GCKeyLeftShift$GCKeyM$GCKeyN$GCKeyNine$GCKeyNonUSBackslash$GCKeyNonUSPound$GCKeyO$GCKeyOne$GCKeyOpenBracket$GCKeyP$GCKeyPageDown$GCKeyPageUp$GCKeyPause$GCKeyPeriod$GCKeyPower$GCKeyPrintScreen$GCKeyQ$GCKeyQuote$GCKeyR$GCKeyReturnOrEnter$GCKeyRightAlt$GCKeyRightArrow$GCKeyRightControl$GCKeyRightGUI$GCKeyRightShift$GCKeyS$GCKeyScrollLock$GCKeySemicolon$GCKeySeven$GCKeySix$GCKeySlash$GCKeySpacebar$GCKeyT$GCKeyTab$GCKeyThree$GCKeyTwo$GCKeyU$GCKeyUpArrow$GCKeyV$GCKeyW$GCKeyX$GCKeyY$GCKeyZ$GCKeyZero$GCKeyboardDidConnectNotification$GCKeyboardDidDisconnectNotification$GCMouseDidBecomeCurrentNotification$GCMouseDidConnectNotification$GCMouseDidDisconnectNotification$GCMouseDidStopBeingCurrentNotification$GCProductCategoryCoalescedRemote$GCProductCategoryControlCenterRemote$GCProductCategoryDualSense$GCProductCategoryDualShock4$GCProductCategoryHID$GCProductCategoryKeyboard$GCProductCategoryMFi$GCProductCategoryMouse$GCProductCategorySiriRemote1stGen$GCProductCategorySiriRemote2ndGen$GCProductCategoryUniversalElectronicsRemote$GCProductCategoryXboxOne$GCRacingWheelDidConnectNotification$GCRacingWheelDidDisconnectNotification$"""
enums = """$GCControllerPlayerIndex1@0$GCControllerPlayerIndex2@1$GCControllerPlayerIndex3@2$GCControllerPlayerIndex4@3$GCControllerPlayerIndexUnset@-1$GCDeviceBatteryStateCharging@1$GCDeviceBatteryStateDischarging@0$GCDeviceBatteryStateFull@2$GCDeviceBatteryStateUnknown@-1$GCDevicePhysicalInputElementChanged@1$GCDevicePhysicalInputElementNoChange@0$GCDevicePhysicalInputElementUnknownChange@-1$GCDualSenseAdaptiveTriggerDiscretePositionCount@10$GCDualSenseAdaptiveTriggerModeFeedback@1$GCDualSenseAdaptiveTriggerModeOff@0$GCDualSenseAdaptiveTriggerModeSlopeFeedback@4$GCDualSenseAdaptiveTriggerModeVibration@3$GCDualSenseAdaptiveTriggerModeWeapon@2$GCDualSenseAdaptiveTriggerStatusFeedbackLoadApplied@1$GCDualSenseAdaptiveTriggerStatusFeedbackNoLoad@0$GCDualSenseAdaptiveTriggerStatusSlopeFeedbackApplyingLoad@8$GCDualSenseAdaptiveTriggerStatusSlopeFeedbackFinished@9$GCDualSenseAdaptiveTriggerStatusSlopeFeedbackReady@7$GCDualSenseAdaptiveTriggerStatusUnknown@-1$GCDualSenseAdaptiveTriggerStatusVibrationIsVibrating@6$GCDualSenseAdaptiveTriggerStatusVibrationNotVibrating@5$GCDualSenseAdaptiveTriggerStatusWeaponFired@4$GCDualSenseAdaptiveTriggerStatusWeaponFiring@3$GCDualSenseAdaptiveTriggerStatusWeaponReady@2$GCExtendedGamepadSnapshotDataVersion1@256$GCExtendedGamepadSnapshotDataVersion2@257$GCMicroGamepadSnapshotDataVersion1@256$GCSystemGestureStateAlwaysReceive@1$GCSystemGestureStateDisabled@2$GCSystemGestureStateEnabled@0$GCTouchStateDown@1$GCTouchStateMoving@2$GCTouchStateUp@0$"""
misc.update(
    {
        "GCDevicePhysicalInputElementChange": NewType(
            "GCDevicePhysicalInputElementChange", int
        ),
        "GCDeviceBatteryState": NewType("GCDeviceBatteryState", int),
        "GCControllerPlayerIndex": NewType("GCControllerPlayerIndex", int),
        "GCMicroGamepadSnapshotDataVersion": NewType(
            "GCMicroGamepadSnapshotDataVersion", int
        ),
        "GCDualSenseAdaptiveTriggerStatus": NewType(
            "GCDualSenseAdaptiveTriggerStatus", int
        ),
        "GCSystemGestureState": NewType("GCSystemGestureState", int),
        "GCExtendedGamepadSnapshotDataVersion": NewType(
            "GCExtendedGamepadSnapshotDataVersion", int
        ),
        "GCDualSenseAdaptiveTriggerMode": NewType(
            "GCDualSenseAdaptiveTriggerMode", int
        ),
        "GCTouchState": NewType("GCTouchState", int),
    }
)
misc.update(
    {
        "GCInputButtonName": NewType("GCInputButtonName", str),
        "GCHapticsLocality": NewType("GCHapticsLocality", str),
        "GCKeyCode": NewType("GCKeyCode", int),
        "GCInputDirectionPadName": NewType("GCInputDirectionPadName", str),
        "GCInputElementName": NewType("GCInputElementName", str),
        "GCInputAxisName": NewType("GCInputAxisName", str),
    }
)
misc.update({"kIOHIDGCSyntheticDeviceKey": b"GCSyntheticDevice"})
functions = {
    "GCExtendedGamepadSnapshotDataFromNSData": (
        b"Z^{_GCExtendedGamepadSnapshotData=SSffffffffffffffZZZ}@",
        "",
        {"arguments": {0: {"type_modifier": "o"}}},
    ),
    "NSDataFromGCMicroGamepadSnapShotDataV100": (
        b"@^{_GCMicroGamepadSnapShotDataV100=SSffff}",
        "",
        {"arguments": {0: {"type_modifier": "n"}}},
    ),
    "GCGamepadSnapShotDataV100FromNSData": (
        b"Z^{_GCGamepadSnapShotDataV100=SSffffffff}@",
        "",
        {"arguments": {0: {"type_modifier": "o"}}},
    ),
    "NSDataFromGCMicroGamepadSnapshotData": (
        b"@^{_GCMicroGamepadSnapshotData=SSffff}",
        "",
        {"arguments": {0: {"type_modifier": "n"}}},
    ),
    "NSDataFromGCGamepadSnapShotDataV100": (
        b"@^{_GCGamepadSnapShotDataV100=SSffffffff}",
        "",
        {"arguments": {0: {"type_modifier": "n"}}},
    ),
    "GCMicroGamepadSnapshotDataFromNSData": (
        b"Z^{_GCMicroGamepadSnapshotData=SSffff}@",
        "",
        {"arguments": {0: {"type_modifier": "o"}}},
    ),
    "GCExtendedGamepadSnapShotDataV100FromNSData": (
        b"Z^{_GCExtendedGamepadSnapShotDataV100=SSffffffffffffff}@",
        "",
        {"arguments": {0: {"type_modifier": "o"}}},
    ),
    "GCMicroGamepadSnapShotDataV100FromNSData": (
        b"Z^{_GCMicroGamepadSnapShotDataV100=SSffff}@",
        "",
        {"arguments": {0: {"type_modifier": "o"}}},
    ),
    "NSDataFromGCExtendedGamepadSnapShotDataV100": (
        b"@^{_GCExtendedGamepadSnapShotDataV100=SSffffffffffffff}",
    ),
    "NSDataFromGCExtendedGamepadSnapshotData": (
        b"@^{_GCExtendedGamepadSnapshotData=SSffffffffffffffZZZ}",
        "",
        {"arguments": {0: {"type_modifier": "n"}}},
    ),
}
r = objc.registerMetaDataForSelector
objc._updatingMetadata(True)
try:
    r(
        b"GCController",
        b"controllerPausedHandler",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                }
            }
        },
    )
    r(b"GCController", b"isAttachedToDevice", {"retval": {"type": b"Z"}})
    r(b"GCController", b"isSnapshot", {"retval": {"type": b"Z"}})
    r(
        b"GCController",
        b"setControllerPausedHandler:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                    }
                }
            }
        },
    )
    r(
        b"GCController",
        b"setShouldMonitorBackgroundEvents",
        {"arguments": {2: {"type": "Z"}}},
    )
    r(
        b"GCController",
        b"setShouldMonitorBackgroundEvents:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(b"GCController", b"shouldMonitorBackgroundEvents", {"retval": {"type": b"Z"}})
    r(
        b"GCController",
        b"startWirelessControllerDiscoveryWithCompletionHandler:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}},
                    }
                }
            }
        },
    )
    r(b"GCController", b"supportsHIDDevice:", {"retval": {"type": b"Z"}})
    r(
        b"GCControllerAxisInput",
        b"setValueChangedHandler:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"f"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"GCControllerAxisInput",
        b"valueChangedHandler",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"f"},
                    },
                }
            }
        },
    )
    r(b"GCControllerButtonInput", b"isPressed", {"retval": {"type": b"Z"}})
    r(b"GCControllerButtonInput", b"isTouched", {"retval": {"type": b"Z"}})
    r(
        b"GCControllerButtonInput",
        b"pressedChangedHandler",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"f"},
                        3: {"type": b"Z"},
                    },
                }
            }
        },
    )
    r(
        b"GCControllerButtonInput",
        b"setPressedChangedHandler:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"f"},
                            3: {"type": b"Z"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"GCControllerButtonInput",
        b"setTouchedChangedHandler:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"f"},
                            3: {"type": b"Z"},
                            4: {"type": b"Z"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"GCControllerButtonInput",
        b"setValueChangedHandler:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"f"},
                            3: {"type": b"Z"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"GCControllerButtonInput",
        b"touchedChangedHandler",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"f"},
                        3: {"type": b"Z"},
                        4: {"type": b"Z"},
                    },
                }
            }
        },
    )
    r(
        b"GCControllerButtonInput",
        b"valueChangedHandler",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"f"},
                        3: {"type": b"Z"},
                    },
                }
            }
        },
    )
    r(
        b"GCControllerDirectionPad",
        b"setValueChangedHandler:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"f"},
                            3: {"type": b"f"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"GCControllerDirectionPad",
        b"valueChangedHandler",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"f"},
                        3: {"type": b"f"},
                    },
                }
            }
        },
    )
    r(b"GCControllerElement", b"isAnalog", {"retval": {"type": b"Z"}})
    r(b"GCControllerElement", b"isBoundToSystemGesture", {"retval": {"type": b"Z"}})
    r(
        b"GCControllerTouchpad",
        b"reportsAbsoluteTouchSurfaceValues",
        {"retval": {"type": b"Z"}},
    )
    r(
        b"GCControllerTouchpad",
        b"setReportsAbsoluteTouchSurfaceValues:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"GCControllerTouchpad",
        b"setTouchDown:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"f"},
                            2: {"type": b"f"},
                            3: {"type": b"f"},
                            4: {"type": b"Z"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"GCControllerTouchpad",
        b"setTouchMoved:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"f"},
                            2: {"type": b"f"},
                            3: {"type": b"f"},
                            4: {"type": b"Z"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"GCControllerTouchpad",
        b"setTouchUp:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"f"},
                            2: {"type": b"f"},
                            3: {"type": b"f"},
                            4: {"type": b"Z"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"GCControllerTouchpad",
        b"setValueForXAxis:yAxis:touchDown:buttonValue:",
        {"arguments": {4: {"type": b"Z"}}},
    )
    r(
        b"GCControllerTouchpad",
        b"touchDown",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"f"},
                        2: {"type": b"f"},
                        3: {"type": b"f"},
                        4: {"type": b"Z"},
                    },
                }
            }
        },
    )
    r(
        b"GCControllerTouchpad",
        b"touchMoved",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"f"},
                        2: {"type": b"f"},
                        3: {"type": b"f"},
                        4: {"type": b"Z"},
                    },
                }
            }
        },
    )
    r(
        b"GCControllerTouchpad",
        b"touchUp",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"f"},
                        2: {"type": b"f"},
                        3: {"type": b"f"},
                        4: {"type": b"Z"},
                    },
                }
            }
        },
    )
    r(
        b"GCDualSenseAdaptiveTrigger",
        b"setModeFeedbackWithResistiveStrengths:",
        {
            "arguments": {
                2: {
                    "type": b"{_GCDualSenseAdaptiveTriggerPositionalResistiveStrengths=[10f]}"
                }
            }
        },
    )
    r(
        b"GCDualSenseAdaptiveTrigger",
        b"setModeVibrationWithAmplitudes:frequency:",
        {
            "arguments": {
                2: {"type": b"{_GCDualSenseAdaptiveTriggerPositionalAmplitudes=[10f]}"}
            }
        },
    )
    r(
        b"GCEventViewController",
        b"controllerUserInteractionEnabled",
        {"retval": {"type": "Z"}},
    )
    r(
        b"GCEventViewController",
        b"setControllerUserInteractionEnabled:",
        {"arguments": {2: {"type": "Z"}}},
    )
    r(
        b"GCExtendedGamepad",
        b"setValueChangedHandler:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"GCExtendedGamepad",
        b"valueChangedHandler",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"@"},
                    },
                }
            }
        },
    )
    r(
        b"GCGamepad",
        b"setValueChangedHandler:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"GCGamepad",
        b"valueChangedHandler",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"@"},
                    },
                }
            }
        },
    )
    r(b"GCKeyboardInput", b"isAnyKeyPressed", {"retval": {"type": b"Z"}})
    r(
        b"GCKeyboardInput",
        b"keyChangedHandler",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"@"},
                        3: {"type": b"q"},
                        4: {"type": b"Z"},
                    },
                }
            }
        },
    )
    r(
        b"GCKeyboardInput",
        b"setKeyChangedHandler:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                            3: {"type": b"q"},
                            4: {"type": b"Z"},
                        },
                    }
                }
            }
        },
    )
    r(b"GCMicroGamepad", b"allowsRotation", {"retval": {"type": "Z"}})
    r(b"GCMicroGamepad", b"reportsAbsoluteDpadValues", {"retval": {"type": "Z"}})
    r(b"GCMicroGamepad", b"setAllowsRotation:", {"arguments": {2: {"type": "Z"}}})
    r(
        b"GCMicroGamepad",
        b"setReportsAbsoluteDpadValues:",
        {"arguments": {2: {"type": "Z"}}},
    )
    r(
        b"GCMicroGamepad",
        b"setValueChangedHandler:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"GCMicroGamepad",
        b"valueChangedHandler",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"@"},
                    },
                }
            }
        },
    )
    r(b"GCMotion", b"acceleration", {"retval": {"type": b"{_GCAcceleration=ddd}"}})
    r(b"GCMotion", b"gravity", {"retval": {"type": b"{_GCAcceleration=ddd}"}})
    r(b"GCMotion", b"hasAttitude", {"retval": {"type": b"Z"}})
    r(b"GCMotion", b"hasAttitudeAndRotationRate", {"retval": {"type": "Z"}})
    r(b"GCMotion", b"hasGravityAndUserAcceleration", {"retval": {"type": "Z"}})
    r(b"GCMotion", b"hasRotationRate", {"retval": {"type": b"Z"}})
    r(b"GCMotion", b"rotationRate", {"retval": {"type": b"{_GCRotationRate=ddd}"}})
    r(b"GCMotion", b"sensorsActive", {"retval": {"type": b"Z"}})
    r(b"GCMotion", b"sensorsRequireManualActivation", {"retval": {"type": b"Z"}})
    r(
        b"GCMotion",
        b"setAcceleration:",
        {"arguments": {2: {"type": b"{_GCAcceleration=ddd}"}}},
    )
    r(
        b"GCMotion",
        b"setGravity:",
        {"arguments": {2: {"type": b"{_GCAcceleration=ddd}"}}},
    )
    r(
        b"GCMotion",
        b"setRotationRate:",
        {"arguments": {2: {"type": b"{_GCRotationRate=ddd}"}}},
    )
    r(b"GCMotion", b"setSensorsActive:", {"arguments": {2: {"type": b"Z"}}})
    r(
        b"GCMotion",
        b"setUserAcceleration:",
        {"arguments": {2: {"type": b"{_GCAcceleration=ddd}"}}},
    )
    r(
        b"GCMotion",
        b"setValueChangedHandler:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                    }
                }
            }
        },
    )
    r(b"GCMotion", b"userAcceleration", {"retval": {"type": b"{_GCAcceleration=ddd}"}})
    r(
        b"GCMotion",
        b"valueChangedHandler",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                }
            }
        },
    )
    r(
        b"GCMouseInput",
        b"mouseMovedHandler",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"f"},
                        3: {"type": b"f"},
                    },
                }
            }
        },
    )
    r(
        b"GCMouseInput",
        b"setMouseMovedHandler:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"f"},
                            3: {"type": b"f"},
                        },
                    }
                }
            }
        },
    )
    r(b"GCPhysicalInputProfile", b"hasRemappedElements", {"retval": {"type": "Z"}})
    r(
        b"GCPhysicalInputProfile",
        b"setValueDidChangeHandler:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"GCPhysicalInputProfile",
        b"valueDidChangeHandler",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"@"},
                    },
                }
            }
        },
    )
    r(
        b"GCRacingWheel",
        b"acquireDeviceWithError:",
        {"retval": {"type": b"Z"}, "arguments": {2: {"type_modifier": b"o"}}},
    )
    r(b"GCRacingWheel", b"isAcquired", {"retval": {"type": b"Z"}})
    r(b"GCRacingWheel", b"isSnapshot", {"retval": {"type": b"Z"}})
    r(b"NSObject", b"absoluteInput", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"aliases", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"axes", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"buttons", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"canWrap", {"required": True, "retval": {"type": b"Z"}})
    r(b"NSObject", b"capture", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"changeForElement:",
        {"required": True, "retval": {"type": b"q"}, "arguments": {2: {"type": b"@"}}},
    )
    r(b"NSObject", b"changedElements", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"delta", {"required": True, "retval": {"type": b"f"}})
    r(
        b"NSObject",
        b"deltaDidChangeHandler",
        {
            "required": True,
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"@"},
                        3: {"type": b"f"},
                    },
                },
                "type": b"@?",
            },
        },
    )
    r(b"NSObject", b"device", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"down", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"dpads", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"elementValueDidChangeHandler",
        {
            "required": True,
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"@"},
                    },
                },
                "type": b"@?",
            },
        },
    )
    r(b"NSObject", b"elements", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"handlerQueue", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"inputStateAvailableHandler",
        {
            "required": True,
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                },
                "type": b"@?",
            },
        },
    )
    r(
        b"NSObject",
        b"inputStateQueueDepth",
        {"required": True, "retval": {"type": b"q"}},
    )
    r(b"NSObject", b"isAnalog", {"required": True, "retval": {"type": b"Z"}})
    r(b"NSObject", b"isPressed", {"required": True, "retval": {"type": b"Z"}})
    r(b"NSObject", b"isSequential", {"required": True, "retval": {"type": b"Z"}})
    r(b"NSObject", b"isTouched", {"required": True, "retval": {"type": b"Z"}})
    r(b"NSObject", b"lastDeltaLatency", {"required": True, "retval": {"type": b"d"}})
    r(b"NSObject", b"lastDeltaTimestamp", {"required": True, "retval": {"type": b"d"}})
    r(b"NSObject", b"lastEventLatency", {"required": True, "retval": {"type": b"d"}})
    r(b"NSObject", b"lastEventTimestamp", {"required": True, "retval": {"type": b"d"}})
    r(b"NSObject", b"lastPositionLatency", {"required": True, "retval": {"type": b"d"}})
    r(
        b"NSObject",
        b"lastPositionTimestamp",
        {"required": True, "retval": {"type": b"d"}},
    )
    r(
        b"NSObject",
        b"lastPressedStateLatency",
        {"required": True, "retval": {"type": b"d"}},
    )
    r(
        b"NSObject",
        b"lastPressedStateTimestamp",
        {"required": True, "retval": {"type": b"d"}},
    )
    r(
        b"NSObject",
        b"lastTouchedStateLatency",
        {"required": True, "retval": {"type": b"d"}},
    )
    r(
        b"NSObject",
        b"lastTouchedStateTimestamp",
        {"required": True, "retval": {"type": b"d"}},
    )
    r(b"NSObject", b"lastValueLatency", {"required": True, "retval": {"type": b"d"}})
    r(b"NSObject", b"lastValueTimestamp", {"required": True, "retval": {"type": b"d"}})
    r(b"NSObject", b"left", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"localizedName", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"nextInputState", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"objectForKeyedSubscript:",
        {"required": True, "retval": {"type": b"@"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"physicalInputProfile",
        {"required": True, "retval": {"type": b"@"}},
    )
    r(b"NSObject", b"position", {"required": True, "retval": {"type": b"q"}})
    r(
        b"NSObject",
        b"positionDidChangeHandler",
        {
            "required": True,
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"@"},
                        3: {"type": b"q"},
                    },
                },
                "type": b"@?",
            },
        },
    )
    r(b"NSObject", b"positionInput", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"positionRange",
        {"required": True, "retval": {"type": b"{_NSRange=QQ}"}},
    )
    r(
        b"NSObject",
        b"pressedDidChangeHandler",
        {
            "required": True,
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"@"},
                        3: {"type": b"Z"},
                    },
                },
                "type": b"@?",
            },
        },
    )
    r(b"NSObject", b"pressedInput", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"productCategory", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"relativeInput", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"right", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"setDeltaDidChangeHandler:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                            3: {"type": b"f"},
                        },
                    },
                    "type": b"@?",
                }
            },
        },
    )
    r(
        b"NSObject",
        b"setElementValueDidChangeHandler:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                        },
                    },
                    "type": b"@?",
                }
            },
        },
    )
    r(
        b"NSObject",
        b"setHandlerQueue:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"setInputStateAvailableHandler:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                    },
                    "type": b"@?",
                }
            },
        },
    )
    r(
        b"NSObject",
        b"setInputStateQueueDepth:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"q"}}},
    )
    r(
        b"NSObject",
        b"setPositionDidChangeHandler:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                            3: {"type": b"q"},
                        },
                    },
                    "type": b"@?",
                }
            },
        },
    )
    r(
        b"NSObject",
        b"setPressedDidChangeHandler:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                            3: {"type": b"Z"},
                        },
                    },
                    "type": b"@?",
                }
            },
        },
    )
    r(
        b"NSObject",
        b"setTouchedDidChangeHandler:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                            3: {"type": b"Z"},
                        },
                    },
                    "type": b"@?",
                }
            },
        },
    )
    r(
        b"NSObject",
        b"setValueDidChangeHandler:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                            3: {"type": b"f"},
                        },
                    },
                    "type": b"@?",
                }
            },
        },
    )
    r(b"NSObject", b"sfSymbolsName", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"switches", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"touchedDidChangeHandler",
        {
            "required": True,
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"@"},
                        3: {"type": b"Z"},
                    },
                },
                "type": b"@?",
            },
        },
    )
    r(b"NSObject", b"touchedInput", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"up", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"value", {"required": True, "retval": {"type": b"f"}})
    r(
        b"NSObject",
        b"valueDidChangeHandler",
        {
            "required": True,
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {
                        0: {"type": b"^v"},
                        1: {"type": b"@"},
                        2: {"type": b"@"},
                        3: {"type": b"f"},
                    },
                },
                "type": b"@?",
            },
        },
    )
    r(b"NSObject", b"vendorName", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"xAxis", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"yAxis", {"required": True, "retval": {"type": b"@"}})
finally:
    objc._updatingMetadata(False)
expressions = {}

# END OF FILE



from PyObjCTools.TestSupport import TestCase, min_os_level
import GameController


class TestGCMouseInput(TestCase):
    @min_os_level("11.0")
    def test_methods11_0(self):
        self.assertResultIsBlock(GameController.GCMouseInput.mouseMovedHandler, b"v@ff")
        self.assertArgIsBlock(
            GameController.GCMouseInput.setMouseMovedHandler_, 0, b"v@ff"
        )



from PyObjCTools.TestSupport import (
    TestCase,
    min_sdk_level,
)
import GameController


class TestGCTouchedStateInputHelper(GameController.NSObject):
    def touchedDidChangeHandler(self):
        return 1

    def setTouchedDidChangeHandler_(self, a):
        pass

    def isTouched(self):
        return 1

    def lastTouchedStateTimestamp(self):
        return 1

    def lastTouchedStateLatency(self):
        return 1


class TestGCTouchedStateInput(TestCase):
    @min_sdk_level("13.0")
    def test_protocols(self):
        self.assertProtocolExists("GCTouchedStateInput")

    def test_protocol_methods(self):
        self.assertResultIsBlock(
            TestGCTouchedStateInputHelper.touchedDidChangeHandler, b"v@@Z"
        )
        self.assertArgIsBlock(
            TestGCTouchedStateInputHelper.setTouchedDidChangeHandler_, 0, b"v@@Z"
        )
        self.assertResultIsBOOL(TestGCTouchedStateInputHelper.isTouched)
        self.assertResultHasType(
            TestGCTouchedStateInputHelper.lastTouchedStateTimestamp, b"d"
        )
        self.assertResultHasType(
            TestGCTouchedStateInputHelper.lastTouchedStateLatency, b"d"
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import objc
import GameController


class TestGCMotion(TestCase):
    @min_os_level("10.10")
    def testClasses(self):
        self.assertIsInstance(GameController.GCMotion, objc.objc_class)

    @min_os_level("10.10")
    def test_structs(self):
        v = GameController.GCAcceleration()
        self.assertIsInstance(v.x, float)
        self.assertIsInstance(v.y, float)
        self.assertIsInstance(v.z, float)
        self.assertPickleRoundTrips(v)

        v = GameController.GCRotationRate()
        self.assertIsInstance(v.x, float)
        self.assertIsInstance(v.y, float)
        self.assertIsInstance(v.z, float)
        self.assertPickleRoundTrips(v)

        v = GameController.GCQuaternion()
        self.assertIsInstance(v.x, float)
        self.assertIsInstance(v.y, float)
        self.assertIsInstance(v.z, float)
        self.assertIsInstance(v.w, float)
        self.assertPickleRoundTrips(v)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBlock(GameController.GCMotion.valueChangedHandler, b"v@")
        self.assertArgIsBlock(GameController.GCMotion.setValueChangedHandler_, 0, b"v@")

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertResultIsBOOL(GameController.GCMotion.hasAttitudeAndRotationRate)

    @min_os_level("11.0")
    def testMethods11_0(self):
        self.assertResultIsBOOL(GameController.GCMotion.sensorsRequireManualActivation)

        self.assertResultIsBOOL(GameController.GCMotion.sensorsActive)
        self.assertArgIsBOOL(GameController.GCMotion.setSensorsActive_, 0)

        self.assertResultIsBOOL(GameController.GCMotion.hasAttitude)
        self.assertResultIsBOOL(GameController.GCMotion.hasRotationRate)

        self.assertResultIsBOOL(GameController.GCMotion.hasGravityAndUserAcceleration)



from PyObjCTools.TestSupport import TestCase, min_os_level
import GameController
import objc


class TestGCKeyboardInput(TestCase):
    @min_os_level("11.0")
    def test_methods11_0(self):
        self.assertResultIsBlock(
            GameController.GCKeyboardInput.keyChangedHandler,
            b"v@@" + objc._C_CFIndex + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            GameController.GCKeyboardInput.setKeyChangedHandler_,
            0,
            b"v@@" + objc._C_CFIndex + objc._C_NSBOOL,
        )

        self.assertResultIsBOOL(GameController.GCKeyboardInput.isAnyKeyPressed)



from PyObjCTools.TestSupport import (
    TestCase,
    min_sdk_level,
)
import GameController  # noqa: F401


class TestGCAxisElement(TestCase):
    @min_sdk_level("13.0")
    def test_protocols(self):
        self.assertProtocolExists("GCAxisElement")



from PyObjCTools.TestSupport import TestCase, min_os_level
import objc
import GameController


class TestGCExtendedGamepadSnapshot(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(GameController.GCExtendedGamepadSnapshotDataVersion)

    def test_constants(self):
        self.assertEqual(GameController.GCExtendedGamepadSnapshotDataVersion1, 0x0100)
        self.assertEqual(GameController.GCExtendedGamepadSnapshotDataVersion2, 0x0101)

    @min_os_level("10.14.4")
    def test_constants_10_14_4(self):
        self.assertIsInstance(
            GameController.GCCurrentExtendedGamepadSnapshotDataVersion, int
        )

    @min_os_level("10.9")
    def testClasses(self):
        self.assertIsInstance(GameController.GCExtendedGamepadSnapshot, objc.objc_class)

    @min_os_level("10.9")
    def testFunctions(self):
        self.assertResultIsBOOL(
            GameController.GCExtendedGamepadSnapShotDataV100FromNSData
        )
        self.assertArgIsOut(
            GameController.GCExtendedGamepadSnapShotDataV100FromNSData, 0
        )

        self.assertArgIsOut(
            GameController.GCExtendedGamepadSnapShotDataV100FromNSData, 0
        )

    @min_os_level("10.9")
    def test_structs(self):
        self.assertEqual(
            GameController.GCExtendedGamepadSnapShotDataV100.__struct_pack__, 1
        )

        v = GameController.GCExtendedGamepadSnapShotDataV100()
        self.assertIsInstance(v.version, int)
        self.assertIsInstance(v.size, int)
        self.assertIsInstance(v.dpadX, float)
        self.assertIsInstance(v.dpadY, float)
        self.assertIsInstance(v.buttonA, float)
        self.assertIsInstance(v.buttonB, float)
        self.assertIsInstance(v.buttonX, float)
        self.assertIsInstance(v.buttonY, float)
        self.assertIsInstance(v.leftShoulder, float)
        self.assertIsInstance(v.rightShoulder, float)
        self.assertIsInstance(v.leftThumbstickX, float)
        self.assertIsInstance(v.leftThumbstickY, float)
        self.assertIsInstance(v.rightThumbstickX, float)
        self.assertIsInstance(v.rightThumbstickY, float)
        self.assertIsInstance(v.leftTrigger, float)
        self.assertIsInstance(v.rightTrigger, float)
        self.assertPickleRoundTrips(v)

    @min_os_level("10.14.1")
    def test_structs_10_14_1(self):
        # XXX: Introduced in the 10.14.4 SDK
        self.assertEqual(
            GameController.GCExtendedGamepadSnapshotData.__struct_pack__, 1
        )

        v = GameController.GCExtendedGamepadSnapshotData()
        self.assertIsInstance(v.version, int)
        self.assertIsInstance(v.size, int)
        self.assertIsInstance(v.dpadX, float)
        self.assertIsInstance(v.dpadY, float)
        self.assertIsInstance(v.buttonA, float)
        self.assertIsInstance(v.buttonB, float)
        self.assertIsInstance(v.buttonX, float)
        self.assertIsInstance(v.buttonY, float)
        self.assertIsInstance(v.leftShoulder, float)
        self.assertIsInstance(v.rightShoulder, float)
        self.assertIsInstance(v.leftThumbstickX, float)
        self.assertIsInstance(v.leftThumbstickY, float)
        self.assertIsInstance(v.rightThumbstickX, float)
        self.assertIsInstance(v.rightThumbstickY, float)
        self.assertIsInstance(v.leftTrigger, float)
        self.assertIsInstance(v.rightTrigger, float)

        self.assertEqual(v.supportsClickableThumbsticks, False)
        self.assertEqual(v.leftThumbstickButton, False)
        self.assertEqual(v.rightThumbstickButton, False)

    @min_os_level("10.14.4")
    def test_functions_10_14_4(self):
        self.assertArgIsOut(GameController.GCExtendedGamepadSnapshotDataFromNSData, 0)
        self.assertResultIsBOOL(GameController.GCExtendedGamepadSnapshotDataFromNSData)

        self.assertArgIsIn(GameController.NSDataFromGCExtendedGamepadSnapshotData, 0)
